#jinja2: newline_sequence:'\r\n'
<powershell>
$admin = [adsi]("WinNT://./administrator, user")
$admin.PSBase.Invoke("SetPassword", "{{ ansible_ssh_pass }}")
net user Administrator "{{ ansible_ssh_pass }}"
</powershell>
<persist>true</persist>
