#!/bin/bash -xe

# Copyright 2016-2017 Nitor Creations Oy
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

[ "$CF_paramSecretsBucket" ] || CF_paramSecretsBucket="$(ndt cf-get-parameter paramSecretsBucket)"

if [ -z "$1" ]; then
  echo "usage: $0 logout|<name>"
  echo "   Secret must be given in stdin"
  exit 1
fi

if [ "$1" = "logout" ]; then
  exit 0
fi

aws s3 cp - s3://${CF_paramSecretsBucket}/$1 <&0
